@@ -1,21 +1,21 @@
-from __future__ import print_function
-import json
 import os
+
 import numpy as np
-from ..utils import (
-    HeadFile,
-    CellBudgetFile,
-    UcnFile,
-    FormattedHeadFile,
-    ZBNetOutput,
-)
+
+from ..datbase import DataInterface, DataListInterface, DataType
 from ..mbase import BaseModel, ModelInterface
 from ..pakbase import PackageInterface
-from ..datbase import DataType, DataInterface, DataListInterface
-from . import NetCdf, netcdf
-from . import shapefile_utils
-from . import vtk
-
+from ..utils import (
+    CellBudgetFile,
+    FormattedHeadFile,
+    HeadFile,
+    UcnFile,
+    ZBNetOutput,
+    import_optional_dependency,
+)
+from . import NetCdf, netcdf, shapefile_utils, vtk
+from .longnames import NC_LONG_NAMES
+from .unitsformat import NC_UNITS_FORMAT
 
 NC_PRECISION_TYPE = {
     np.float64: "f8",
@@ -25,12 +25,6 @@ NC_PRECISION_TYPE = {
     np.int32: "i4",
 }
 
-path = os.path.split(netcdf.__file__)[0]
-with open(path + "/longnames.json") as f:
-    NC_LONG_NAMES = json.load(f)
-with open(path + "/unitsformat.json") as f:
-    NC_UNITS_FORMAT = json.load(f)
-
 
 def ensemble_helper(
     inputs_filename, outputs_filename, models, add_reals=True, **kwargs
@@ -88,7 +82,7 @@ def ensemble_helper(
             outputs_filename,
             models[0],
             models[0].load_results(as_dict=True),
-            **kwargs
+            **kwargs,
         )
         vdict = {}
         vdicts = [
@@ -96,7 +90,7 @@ def ensemble_helper(
                 vdict,
                 models[0],
                 models[0].load_results(as_dict=True),
-                **kwargs
+                **kwargs,
             )
         ]
         i = 1
@@ -150,7 +144,7 @@ def _add_output_nc_variable(
     mask_array3d=None,
 ):
     if logger:
-        logger.log("creating array for {0}".format(var_name))
+        logger.log(f"creating array for {var_name}")
 
     array = np.zeros(
         (len(times), shape3d[0], shape3d[1], shape3d[2]), dtype=np.float32
@@ -204,7 +198,7 @@ def _add_output_nc_variable(
                     continue
 
     if logger:
-        logger.log("creating array for {0}".format(var_name))
+        logger.log(f"creating array for {var_name}")
 
     for mask_val in mask_vals:
         array[np.where(array == mask_val)] = np.NaN
@@ -239,7 +233,7 @@ def _add_output_nc_variable(
             dimensions=dim_tuple,
         )
     except Exception as e:
-        estr = "error creating variable {0}:\n{1}".format(var_name, str(e))
+        estr = f"error creating variable {var_name}:\n{e!s}"
         if logger:
             logger.lraise(estr)
         else:
@@ -248,9 +242,7 @@ def _add_output_nc_variable(
     try:
         var[:] = array
     except Exception as e:
-        estr = "error setting array to variable {0}:\n{1}".format(
-            var_name, str(e)
-        )
+        estr = f"error setting array to variable {var_name}:\n{e!s}"
         if logger:
             logger.lraise(estr)
         else:
@@ -275,16 +267,16 @@ def _add_output_nc_zonebudget_variable(f, array, var_name, flux, logger=None):
 
     """
     if logger:
-        logger.log("creating array for {}".format(var_name))
+        logger.log(f"creating array for {var_name}")
 
     mn = np.min(array)
     mx = np.max(array)
 
     precision_str = "f4"
     if flux:
-        units = "{}^3/{}".format(f.grid_units, f.time_units)
+        units = f"{f.grid_units}^3/{f.time_units}"
     else:
-        units = "{}^3".format(f.grid_units)
+        units = f"{f.grid_units}^3"
     attribs = {"long_name": var_name}
     attribs["coordinates"] = "time zone"
     attribs["min"] = mn
@@ -340,7 +332,7 @@ def output_helper(f, ml, oudic, **kwargs):
         mask_vals = kwargs.pop("masked_vals")
     if len(kwargs) > 0 and logger is not None:
         str_args = ",".join(kwargs)
-        logger.warn("unused kwargs: " + str_args)
+        logger.warn(f"unused kwargs: {str_args}")
 
     zonebud = None
     zbkey = None
@@ -356,9 +348,7 @@ def output_helper(f, ml, oudic, **kwargs):
     # that they will line up
     for key in oudic.keys():
         out = oudic[key]
-        times = [
-            float("{0:15.6f}".format(t)) for t in out.recordarray["totim"]
-        ]
+        times = [float(f"{t:15.6f}") for t in out.recordarray["totim"]]
         out.recordarray["totim"] = times
 
     times = []
@@ -396,7 +386,7 @@ def output_helper(f, ml, oudic, **kwargs):
     if len(skipped_times) > 0:
         msg = (
             "the following output times are not common to all "
-            "output files and are being skipped:\n{0}".format(skipped_times)
+            f"output files and are being skipped:\n{skipped_times}"
         )
         if logger:
             logger.warn(msg)
@@ -476,7 +466,7 @@ def output_helper(f, ml, oudic, **kwargs):
                     )
 
             else:
-                estr = "unrecognized file extension:{0}".format(filename)
+                estr = f"unrecognized file extension:{filename}"
                 if logger:
                     logger.lraise(estr)
                 else:
@@ -535,7 +525,7 @@ def output_helper(f, ml, oudic, **kwargs):
                             continue
                         if mflay is not None and k != mflay:
                             continue
-                        name = attrib_name + "{}_{}".format(per, k)
+                        name = f"{attrib_name}{per}_{k}"
                         attrib_dict[name] = plotarray[per][k]
 
             elif isinstance(out_obj, CellBudgetFile):
@@ -561,14 +551,14 @@ def output_helper(f, ml, oudic, **kwargs):
                                 continue
                             if mflay is not None and k != mflay:
                                 continue
-                            name = attrib_name + "{}_{}".format(per, k)
+                            name = f"{attrib_name}{per}_{k}"
                             attrib_dict[name] = plotarray[per][k]
 
         if attrib_dict:
             shapefile_utils.write_grid_shapefile(f, ml.modelgrid, attrib_dict)
 
     else:
-        msg = "unrecognized export argument:{0}".format(f)
+        msg = f"unrecognized export argument:{f}"
         if logger:
             logger.lraise(msg)
         else:
@@ -627,28 +617,33 @@ def model_export(f, ml, fmt=None, **kwargs):
 
     elif fmt == "vtk":
         # call vtk model export
-        nanval = kwargs.get("nanval", -1e20)
+        name = kwargs.get("name", ml.name)
+        xml = kwargs.get("xml", False)
+        masked_values = kwargs.get("masked_values", None)
+        pvd = kwargs.get("pvd", False)
         smooth = kwargs.get("smooth", False)
         point_scalars = kwargs.get("point_scalars", False)
-        vtk_grid_type = kwargs.get("vtk_grid_type", "auto")
-        true2d = kwargs.get("true2d", False)
-        binary = kwargs.get("binary", False)
+        binary = kwargs.get("binary", True)
+        vertical_exageration = kwargs.get("vertical_exageration", 1)
         kpers = kwargs.get("kpers", None)
-        vtk.export_model(
+        package_names = kwargs.get("package_names", None)
+
+        vtkobj = vtk.Vtk(
             ml,
-            f,
-            package_names=package_names,
-            nanval=nanval,
+            vertical_exageration=vertical_exageration,
+            binary=binary,
+            xml=xml,
+            pvd=pvd,
             smooth=smooth,
             point_scalars=point_scalars,
-            vtk_grid_type=vtk_grid_type,
-            true2d=true2d,
-            binary=binary,
-            kpers=kpers,
         )
+        vtkobj.add_model(
+            ml, masked_values=masked_values, selpaklist=package_names
+        )
+        vtkobj.write(os.path.join(f, name), kpers)
 
     else:
-        raise NotImplementedError("unrecognized export argument:{0}".format(f))
+        raise NotImplementedError(f"unrecognized export argument:{f}")
 
     return f
 
@@ -701,9 +696,7 @@ def package_export(f, pak, fmt=None, **kwargs):
                         try:
                             f = array2d_export(f, a, **kwargs)
                         except:
-                            f.logger.warn(
-                                "error adding {0} as variable".format(a.name)
-                            )
+                            f.logger.warn(f"error adding {a.name} as variable")
                     elif a.data_type == DataType.array3d:
                         f = array3d_export(f, a, **kwargs)
                     elif a.data_type == DataType.transient2d:
@@ -721,13 +714,30 @@ def package_export(f, pak, fmt=None, **kwargs):
 
     elif fmt == "vtk":
         # call vtk array export to folder
-        nanval = kwargs.get("nanval", -1e20)
+        name = kwargs.get("name", pak.name[0])
+        xml = kwargs.get("xml", False)
+        masked_values = kwargs.get("masked_values", None)
+        pvd = kwargs.get("pvd", False)
         smooth = kwargs.get("smooth", False)
         point_scalars = kwargs.get("point_scalars", False)
-        vtk_grid_type = kwargs.get("vtk_grid_type", "auto")
-        true2d = kwargs.get("true2d", False)
-        binary = kwargs.get("binary", False)
+        binary = kwargs.get("binary", True)
+        vertical_exageration = kwargs.get("vertical_exageration", 1)
         kpers = kwargs.get("kpers", None)
+
+        vtkobj = vtk.Vtk(
+            pak.parent,
+            vertical_exageration=vertical_exageration,
+            binary=binary,
+            xml=xml,
+            pvd=pvd,
+            smooth=smooth,
+            point_scalars=point_scalars,
+        )
+
+        vtkobj.add_package(pak, masked_values=masked_values)
+        vtkobj.write(os.path.join(f, name), kper=kpers)
+
+        """
         vtk.export_package(
             pak.parent,
             pak.name,
@@ -740,9 +750,10 @@ def package_export(f, pak, fmt=None, **kwargs):
             binary=binary,
             kpers=kpers,
         )
+        """
 
     else:
-        raise NotImplementedError("unrecognized export argument:{0}".format(f))
+        raise NotImplementedError(f"unrecognized export argument:{f}")
 
 
 def generic_array_export(
@@ -752,7 +763,7 @@ def generic_array_export(
     dimensions=("time", "layer", "y", "x"),
     precision_str="f4",
     units="unitless",
-    **kwargs
+    **kwargs,
 ):
     """
     Method to export a generic array to NetCdf
@@ -807,7 +818,7 @@ def generic_array_export(
     attribs["min"] = mn
     attribs["max"] = mx
     if np.isnan(attribs["min"]) or np.isnan(attribs["max"]):
-        raise Exception("error processing {0}: all NaNs".format(var_name))
+        raise Exception(f"error processing {var_name}: all NaNs")
     try:
         var = f.create_variable(
             var_name,
@@ -816,15 +827,13 @@ def generic_array_export(
             dimensions=dimensions,
         )
     except Exception as e:
-        estr = "error creating variable {0}:\n{1}".format(var_name, str(e))
+        estr = f"error creating variable {var_name}:\n{e!s}"
         f.logger.warn(estr)
         raise Exception(estr)
     try:
         var[:] = array
     except Exception as e:
-        estr = "error setting array to variable {0}:\n{1}".format(
-            var_name, str(e)
-        )
+        estr = f"error setting array to variable {var_name}:\n{e!s}"
         f.logger.warn(estr)
         raise Exception(estr)
     return f
@@ -884,7 +893,7 @@ def mflist_export(f, mfl, **kwargs):
                     for k in range(array.shape[0]):
                         # aname = name+"{0:03d}_{1:02d}".format(kk, k)
                         n = shapefile_utils.shape_attr_name(name, length=4)
-                        aname = "{}{}{}".format(n, k + 1, int(kk) + 1)
+                        aname = f"{n}{k + 1}{int(kk) + 1}"
                         array_dict[aname] = array[k]
             shapefile_utils.write_grid_shapefile(f, modelgrid, array_dict)
         else:
@@ -918,11 +927,11 @@ def mflist_export(f, mfl, **kwargs):
 
         # for name, array in m4d.items():
         for name, array in mfl.masked_4D_arrays_itr():
-            var_name = base_name + "_" + name
+            var_name = f"{base_name}_{name}"
             if isinstance(f, dict):
                 f[var_name] = array
                 continue
-            f.log("processing {0} attribute".format(name))
+            f.log(f"processing {name} attribute")
 
             units = None
             if var_name in NC_UNITS_FORMAT:
@@ -938,9 +947,7 @@ def mflist_export(f, mfl, **kwargs):
             attribs["min"] = np.nanmin(array)
             attribs["max"] = np.nanmax(array)
             if np.isnan(attribs["min"]) or np.isnan(attribs["max"]):
-                raise Exception(
-                    "error processing {0}: all NaNs".format(var_name)
-                )
+                raise Exception(f"error processing {var_name}: all NaNs")
 
             if units is not None:
                 attribs["units"] = units
@@ -953,9 +960,7 @@ def mflist_export(f, mfl, **kwargs):
                     dimensions=dim_tuple,
                 )
             except Exception as e:
-                estr = "error creating variable {0}:\n{1}".format(
-                    var_name, str(e)
-                )
+                estr = f"error creating variable {var_name}:\n{e!s}"
                 f.logger.warn(estr)
                 raise Exception(estr)
 
@@ -963,16 +968,14 @@ def mflist_export(f, mfl, **kwargs):
             try:
                 var[:] = array
             except Exception as e:
-                estr = "error setting array to variable {0}:\n{1}".format(
-                    var_name, str(e)
-                )
+                estr = f"error setting array to variable {var_name}:\n{e!s}"
                 f.logger.warn(estr)
                 raise Exception(estr)
-            f.log("processing {0} attribute".format(name))
+            f.log(f"processing {name} attribute")
 
         return f
     else:
-        raise NotImplementedError("unrecognized export argument:{0}".format(f))
+        raise NotImplementedError(f"unrecognized export argument:{f}")
 
 
 def transient2d_export(f, t2d, fmt=None, **kwargs):
@@ -1016,9 +1019,7 @@ def transient2d_export(f, t2d, fmt=None, **kwargs):
         array_dict = {}
         for kper in range(t2d.model.modeltime.nper):
             u2d = t2d[kper]
-            name = "{}_{}".format(
-                shapefile_utils.shape_attr_name(u2d.name), kper + 1
-            )
+            name = f"{shapefile_utils.shape_attr_name(u2d.name)}_{kper + 1}"
             array_dict[name] = u2d.array
         shapefile_utils.write_grid_shapefile(f, modelgrid, array_dict)
 
@@ -1077,7 +1078,7 @@ def transient2d_export(f, t2d, fmt=None, **kwargs):
         attribs["min"] = mn
         attribs["max"] = mx
         if np.isnan(attribs["min"]) or np.isnan(attribs["max"]):
-            raise Exception("error processing {0}: all NaNs".format(var_name))
+            raise Exception(f"error processing {var_name}: all NaNs")
         try:
             dim_tuple = ("time",) + f.dimension_names
             var = f.create_variable(
@@ -1087,44 +1088,50 @@ def transient2d_export(f, t2d, fmt=None, **kwargs):
                 dimensions=dim_tuple,
             )
         except Exception as e:
-            estr = "error creating variable {0}:\n{1}".format(var_name, str(e))
+            estr = f"error creating variable {var_name}:\n{e!s}"
             f.logger.warn(estr)
             raise Exception(estr)
         try:
             var[:, 0] = array
         except Exception as e:
-            estr = "error setting array to variable {0}:\n{1}".format(
-                var_name, str(e)
-            )
+            estr = f"error setting array to variable {var_name}:\n{e!s}"
             f.logger.warn(estr)
             raise Exception(estr)
         return f
 
     elif fmt == "vtk":
         name = kwargs.get("name", t2d.name)
-        nanval = kwargs.get("nanval", -1e20)
+        xml = kwargs.get("xml", False)
+        masked_values = kwargs.get("masked_values", None)
+        pvd = kwargs.get("pvd", False)
         smooth = kwargs.get("smooth", False)
         point_scalars = kwargs.get("point_scalars", False)
-        vtk_grid_type = kwargs.get("vtk_grid_type", "auto")
-        true2d = kwargs.get("true2d", False)
-        binary = kwargs.get("binary", False)
+        binary = kwargs.get("binary", True)
+        vertical_exageration = kwargs.get("vertical_exageration", 1)
         kpers = kwargs.get("kpers", None)
-        vtk.export_transient(
+
+        if t2d.array is not None:
+            if hasattr(t2d, "transient_2ds"):
+                d = t2d.transient_2ds
+            else:
+                d = {ix: i for ix, i in enumerate(t2d.array)}
+        else:
+            raise AssertionError("No data available to export")
+
+        vtkobj = vtk.Vtk(
             t2d.model,
-            t2d.array,
-            f,
-            name,
-            nanval=nanval,
+            vertical_exageration=vertical_exageration,
+            binary=binary,
+            xml=xml,
+            pvd=pvd,
             smooth=smooth,
             point_scalars=point_scalars,
-            array2d=True,
-            vtk_grid_type=vtk_grid_type,
-            true2d=true2d,
-            binary=binary,
-            kpers=kpers,
         )
+        vtkobj.add_transient_array(d, name=name, masked_values=masked_values)
+        vtkobj.write(os.path.join(f, name), kper=kpers)
+
     else:
-        raise NotImplementedError("unrecognized export argument:{0}".format(f))
+        raise NotImplementedError(f"unrecognized export argument:{f}")
 
 
 def array3d_export(f, u3d, fmt=None, **kwargs):
@@ -1171,9 +1178,7 @@ def array3d_export(f, u3d, fmt=None, **kwargs):
             else:
                 dname = u2d.name
                 array = u2d.array
-            name = "{}_{}".format(
-                shapefile_utils.shape_attr_name(dname), ilay + 1
-            )
+            name = f"{shapefile_utils.shape_attr_name(dname)}_{ilay + 1}"
             array_dict[name] = array
         shapefile_utils.write_grid_shapefile(f, modelgrid, array_dict)
 
@@ -1251,7 +1256,7 @@ def array3d_export(f, u3d, fmt=None, **kwargs):
         attribs["min"] = mn
         attribs["max"] = mx
         if np.isnan(attribs["min"]) or np.isnan(attribs["max"]):
-            raise Exception("error processing {0}: all NaNs".format(var_name))
+            raise Exception(f"error processing {var_name}: all NaNs")
         try:
             var = f.create_variable(
                 var_name,
@@ -1260,15 +1265,13 @@ def array3d_export(f, u3d, fmt=None, **kwargs):
                 dimensions=f.dimension_names,
             )
         except Exception as e:
-            estr = "error creating variable {0}:\n{1}".format(var_name, str(e))
+            estr = f"error creating variable {var_name}:\n{e!s}"
             f.logger.warn(estr)
             raise Exception(estr)
         try:
             var[:] = array
         except Exception as e:
-            estr = "error setting array to variable {0}:\n{1}".format(
-                var_name, str(e)
-            )
+            estr = f"error setting array to variable {var_name}:\n{e!s}"
             f.logger.warn(estr)
             raise Exception(estr)
         return f
@@ -1276,30 +1279,29 @@ def array3d_export(f, u3d, fmt=None, **kwargs):
     elif fmt == "vtk":
         # call vtk array export to folder
         name = kwargs.get("name", u3d.name)
-        nanval = kwargs.get("nanval", -1e20)
+        xml = kwargs.get("xml", False)
+        masked_values = kwargs.get("masked_values", None)
         smooth = kwargs.get("smooth", False)
         point_scalars = kwargs.get("point_scalars", False)
-        vtk_grid_type = kwargs.get("vtk_grid_type", "auto")
-        true2d = kwargs.get("true2d", False)
-        binary = kwargs.get("binary", False)
+        binary = kwargs.get("binary", True)
+        vertical_exageration = kwargs.get("vertical_exageration", 1)
+
         if isinstance(name, list) or isinstance(name, tuple):
             name = name[0]
 
-        vtk.export_array(
+        vtkobj = vtk.Vtk(
             u3d.model,
-            u3d.array,
-            f,
-            name,
-            nanval=nanval,
             smooth=smooth,
             point_scalars=point_scalars,
-            vtk_grid_type=vtk_grid_type,
-            true2d=true2d,
             binary=binary,
+            xml=xml,
+            vertical_exageration=vertical_exageration,
         )
+        vtkobj.add_array(u3d.array, name, masked_values=masked_values)
+        vtkobj.write(os.path.join(f, name))
 
     else:
-        raise NotImplementedError("unrecognized export argument:{0}".format(f))
+        raise NotImplementedError(f"unrecognized export argument:{f}")
 
 
 def array2d_export(f, u2d, fmt=None, **kwargs):
@@ -1396,7 +1398,7 @@ def array2d_export(f, u2d, fmt=None, **kwargs):
         attribs["min"] = mn
         attribs["max"] = mx
         if np.isnan(attribs["min"]) or np.isnan(attribs["max"]):
-            raise Exception("error processing {0}: all NaNs".format(var_name))
+            raise Exception(f"error processing {var_name}: all NaNs")
         try:
             var = f.create_variable(
                 var_name,
@@ -1405,45 +1407,44 @@ def array2d_export(f, u2d, fmt=None, **kwargs):
                 dimensions=f.dimension_names[1:],
             )
         except Exception as e:
-            estr = "error creating variable {0}:\n{1}".format(var_name, str(e))
+            estr = f"error creating variable {var_name}:\n{e!s}"
             f.logger.warn(estr)
             raise Exception(estr)
         try:
             var[:] = array
         except Exception as e:
-            estr = "error setting array to variable {0}:\n{1}".format(
-                var_name, str(e)
-            )
+            estr = f"error setting array to variable {var_name}:\n{e!s}"
             f.logger.warn(estr)
             raise Exception(estr)
         return f
 
     elif fmt == "vtk":
 
-        # call vtk array export to folder
         name = kwargs.get("name", u2d.name)
-        nanval = kwargs.get("nanval", -1e20)
+        xml = kwargs.get("xml", False)
+        masked_values = kwargs.get("masked_values", None)
         smooth = kwargs.get("smooth", False)
         point_scalars = kwargs.get("point_scalars", False)
-        vtk_grid_type = kwargs.get("vtk_grid_type", "auto")
-        true2d = kwargs.get("true2d", False)
-        binary = kwargs.get("binary", False)
-        vtk.export_array(
+        binary = kwargs.get("binary", True)
+        vertical_exageration = kwargs.get("vertical_exageration", 1)
+
+        if isinstance(name, list) or isinstance(name, tuple):
+            name = name[0]
+
+        vtkobj = vtk.Vtk(
             u2d.model,
-            u2d.array,
-            f,
-            name,
-            nanval=nanval,
             smooth=smooth,
             point_scalars=point_scalars,
-            array2d=True,
-            vtk_grid_type=vtk_grid_type,
-            true2d=true2d,
             binary=binary,
+            xml=xml,
+            vertical_exageration=vertical_exageration,
         )
-
+        array = np.ones((modelgrid.nnodes,)) * np.nan
+        array[0 : u2d.array.size] = np.ravel(u2d.array)
+        vtkobj.add_array(array, name, masked_values=masked_values)
+        vtkobj.write(os.path.join(f, name))
     else:
-        raise NotImplementedError("unrecognized export argument:{0}".format(f))
+        raise NotImplementedError(f"unrecognized export argument:{f}")
 
 
 def export_array(
@@ -1504,38 +1505,36 @@ def export_array(
         a = a.copy()
         a[np.isnan(a)] = nodata
         if modelgrid.angrot != 0:
-            try:
-                from scipy.ndimage import rotate
-            except ImportError:
-                rotate = None
-                print("scipy package required to export rotated grid.")
+            ndimage = import_optional_dependency(
+                "scipy.ndimage",
+                error_message="exporting rotated grids requires SciPy.",
+            )
 
-            if rotate is not None:
-                a = rotate(a, modelgrid.angrot, cval=nodata)
-                height_rot, width_rot = a.shape
-                xmin, ymin, xmax, ymax = modelgrid.extent
-                dx = (xmax - xmin) / width_rot
-                dy = (ymax - ymin) / height_rot
-                cellsize = np.max((dx, dy))
-                xoffset, yoffset = xmin, ymin
+            a = ndimage.rotate(a, modelgrid.angrot, cval=nodata)
+            height_rot, width_rot = a.shape
+            xmin, xmax, ymin, ymax = modelgrid.extent
+            dx = (xmax - xmin) / width_rot
+            dy = (ymax - ymin) / height_rot
+            cellsize = np.max((dx, dy))
+            xoffset, yoffset = xmin, ymin
 
         filename = (
             ".".join(filename.split(".")[:-1]) + ".asc"
         )  # enforce .asc ending
         nrow, ncol = a.shape
         a[np.isnan(a)] = nodata
-        txt = "ncols  {:d}\n".format(ncol)
-        txt += "nrows  {:d}\n".format(nrow)
-        txt += "xllcorner  {:f}\n".format(xoffset)
-        txt += "yllcorner  {:f}\n".format(yoffset)
-        txt += "cellsize  {}\n".format(cellsize)
+        txt = f"ncols  {ncol}\n"
+        txt += f"nrows  {nrow}\n"
+        txt += f"xllcorner  {xoffset:f}\n"
+        txt += f"yllcorner  {yoffset:f}\n"
+        txt += f"cellsize  {cellsize}\n"
         # ensure that nodata fmt consistent w values
-        txt += "NODATA_value  {}\n".format(fmt) % (nodata)
+        txt += f"NODATA_value  {fmt}\n" % (nodata)
         with open(filename, "w") as output:
             output.write(txt)
         with open(filename, "ab") as output:
             np.savetxt(output, a, **kwargs)
-        print("wrote {}".format(filename))
+        print(f"wrote {filename}")
 
     elif filename.lower().endswith(".tif"):
         if (
@@ -1545,21 +1544,19 @@ def export_array(
             or modelgrid.delr[0] != modelgrid.delc[0]
         ):
             raise ValueError("GeoTIFF export require a uniform grid.")
-        try:
-            import rasterio
-            from rasterio import Affine
-        except ImportError:
-            print("GeoTIFF export requires the rasterio package.")
-            return
+        rasterio = import_optional_dependency(
+            "rasterio",
+            error_message="GeoTIFF export requires the rasterio.",
+        )
         dxdy = modelgrid.delc[0]
         # because this is only implemented for a structured grid,
         # we can get the xul and yul coordinate from modelgrid.xvertices(0, 0)
         verts = modelgrid.get_cell_vertices(0, 0)
         xul, yul = verts[0]
         trans = (
-            Affine.translation(xul, yul)
-            * Affine.rotation(modelgrid.angrot)
-            * Affine.scale(dxdy, -dxdy)
+            rasterio.Affine.translation(xul, yul)
+            * rasterio.Affine.rotation(modelgrid.angrot)
+            * rasterio.Affine.scale(dxdy, -dxdy)
         )
 
         # third dimension is the number of bands
@@ -1576,7 +1573,7 @@ def export_array(
         elif a.dtype.name == "float32":
             dtype = rasterio.float32
         else:
-            msg = 'ERROR: invalid dtype "{}"'.format(a.dtype.name)
+            msg = f'ERROR: invalid dtype "{a.dtype.name}"'
             raise TypeError(msg)
 
         meta = {
@@ -1592,7 +1589,7 @@ def export_array(
         meta.update(kwargs)
         with rasterio.open(filename, "w", **meta) as dst:
             dst.write(a)
-        print("wrote {}".format(filename))
+        print(f"wrote {filename}")
 
     elif filename.lower().endswith(".shp"):
         from ..export.shapefile_utils import write_grid_shapefile
@@ -1618,7 +1615,7 @@ def export_contours(
     fieldname="level",
     epsg=None,
     prj=None,
-    **kwargs
+    **kwargs,
 ):
     """
     Convert matplotlib contour plot object to shapefile.
@@ -1675,8 +1672,7 @@ def export_contourf(
     filename, contours, fieldname="level", epsg=None, prj=None, **kwargs
 ):
     """
-    Write matplotlib filled contours to shapefile.  This utility requires
-    that shapely is installed.
+    Write matplotlib filled contours to shapefile.
 
     Parameters
     ----------
@@ -1709,16 +1705,10 @@ def export_contourf(
     >>> export_contourf('myfilledcontours.shp', cs)
 
     """
-
-    try:
-        from shapely import geometry
-    except ImportError:
-        raise ImportError("export_contourf requires python shapely package")
-
-    from ..utils.geometry import Polygon
+    from ..utils.geometry import Polygon, is_clockwise
     from .shapefile_utils import recarray2shp
 
-    shapelygeoms = []
+    geoms = []
     level = []
 
     if not isinstance(contours, list):
@@ -1729,33 +1719,36 @@ def export_contourf(
         for idx, col in enumerate(c.collections):
             # Loop through all polygons that have the same intensity level
             for contour_path in col.get_paths():
-                # Create the polygon for this intensity level
-                # The first polygon in the path is the main one, the following
-                # ones are "holes"
+                # Create the polygon(s) for this intensity level
+                poly = None
                 for ncp, cp in enumerate(contour_path.to_polygons()):
                     x = cp[:, 0]
                     y = cp[:, 1]
-                    new_shape = geometry.Polygon(
-                        [(i[0], i[1]) for i in zip(x, y)]
-                    )
+                    verts = [(i[0], i[1]) for i in zip(x, y)]
+                    new_shape = Polygon(verts)
+
                     if ncp == 0:
                         poly = new_shape
                     else:
-                        # Remove the holes if there are any
-                        poly = poly.difference(new_shape)
+                        # check if this is a multipolygon by checking vertex
+                        # order.
+                        if is_clockwise(verts):
+                            # Clockwise is a hole, set to interiors
+                            if not poly.interiors:
+                                poly.interiors = [new_shape.exterior]
+                            else:
+                                poly.interiors.append(new_shape.exterior)
+                        else:
+                            geoms.append(poly)
+                            level.append(levels[idx])
+                            poly = new_shape
 
-                # store shapely geometry object
-                shapelygeoms.append(poly)
-                level.append(levels[idx])
+                if poly is not None:
+                    # store geometry object
+                    geoms.append(poly)
+                    level.append(levels[idx])
 
-    geoms = []
-    for shpgeom in shapelygeoms:
-        xa, ya = shpgeom.exterior.coords.xy
-        interiors = [s.coords for s in shpgeom.interiors]
-        pg = Polygon([(x, y) for x, y in zip(xa, ya)], interiors=interiors)
-        geoms += [pg]
-
-    print("Writing {} polygons".format(len(level)))
+    print(f"Writing {len(level)} polygons")
 
     # Create recarray
     ra = np.array(level, dtype=[(fieldname, float)]).view(np.recarray)
@@ -1774,7 +1767,7 @@ def export_array_contours(
     maxlevels=1000,
     epsg=None,
     prj=None,
-    **kwargs
+    **kwargs,
 ):
     """
     Contour an array using matplotlib; write shapefile of contours.
@@ -1802,11 +1795,7 @@ def export_array_contours(
     **kwargs : keyword arguments to flopy.export.shapefile_utils.recarray2shp
 
     """
-    try:
-        import matplotlib.pyplot as plt
-    except:
-        err_msg = "matplotlib must be installed to use export_array_contours()"
-        raise ImportError(err_msg)
+    import matplotlib.pyplot as plt
 
     if epsg is None:
         epsg = modelgrid.epsg
@@ -1817,9 +1806,7 @@ def export_array_contours(
         imin = np.nanmin(a)
         imax = np.nanmax(a)
         nlevels = np.round(np.abs(imax - imin) / interval, 2)
-        msg = "{:.0f} levels at interval of {} > maxlevels={}".format(
-            nlevels, interval, maxlevels
-        )
+        msg = f"{nlevels:.0f} levels at interval of {interval} > maxlevels={maxlevels}"
         assert nlevels < maxlevels, msg
         levels = np.arange(imin, imax, interval)
     ax = plt.subplots()[-1]
